"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const code_file_loader_1 = require("@graphql-toolkit/code-file-loader");
const git_loader_1 = require("@graphql-toolkit/git-loader");
const github_loader_1 = require("@graphql-toolkit/github-loader");
const apollo_engine_loader_1 = require("@graphql-toolkit/apollo-engine-loader");
const prisma_loader_1 = require("@graphql-toolkit/prisma-loader");
const core_1 = require("@graphql-inspector/core");
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const log_symbols_1 = tslib_1.__importDefault(require("log-symbols"));
const DiffExtension = api => {
    // Schema
    api.loaders.schema.register(new code_file_loader_1.CodeFileLoader());
    api.loaders.schema.register(new git_loader_1.GitLoader());
    api.loaders.schema.register(new github_loader_1.GithubLoader());
    api.loaders.schema.register(new apollo_engine_loader_1.ApolloEngineLoader());
    api.loaders.schema.register(new prisma_loader_1.PrismaLoader());
    return {
        name: 'diff'
    };
};
function hasBreaking(changes) {
    return changes.some(c => c.criticality.level === core_1.CriticalityLevel.Breaking);
}
function renderChange(change) {
    return [getSymbol(change.criticality.level), chalk_1.default.bold(change.message)];
}
function getSymbol(level) {
    const symbols = {
        [core_1.CriticalityLevel.Dangerous]: log_symbols_1.default.warning,
        [core_1.CriticalityLevel.Breaking]: log_symbols_1.default.error,
        [core_1.CriticalityLevel.NonBreaking]: log_symbols_1.default.success
    };
    return symbols[level];
}
exports.plugin = {
    init({ program, loadProjectConfig, reportError }) {
        program
            .command('diff [baseSchema]')
            .action(async (baseSchemaPtr) => {
            try {
                const config = await loadProjectConfig({
                    extensions: [DiffExtension]
                });
                if (!baseSchemaPtr) {
                    const diffConfig = await config.extension('diff');
                    baseSchemaPtr = diffConfig.baseSchema || 'git:origin/master:schema.graphql';
                }
                const [baseSchema, currentSchema] = await Promise.all([
                    config.loadSchema(baseSchemaPtr),
                    config.getSchema(),
                ]);
                const changes = core_1.diff(baseSchema, currentSchema);
                if (!changes.length) {
                    console.log(log_symbols_1.default.success, 'No changes detected');
                }
                else {
                    console.warn(log_symbols_1.default.warning, `Detected the following changes (${changes.length}) between schemas:\n`);
                    changes.forEach(change => {
                        console.log(...renderChange(change));
                    });
                    if (hasBreaking(changes)) {
                        const breakingCount = changes.filter(c => c.criticality.level === core_1.CriticalityLevel.Breaking).length;
                        throw `Detected ${breakingCount} breaking change${breakingCount > 1 ? 's' : ''}\n`;
                    }
                    else {
                        console.log(log_symbols_1.default.success, 'No breaking changes detected\n');
                    }
                    process.exit(0);
                }
            }
            catch (e) {
                reportError(e);
            }
        });
    }
};
//# sourceMappingURL=index.js.map